
/*!
 * EyzyTreeAPI v0.0.29
 * (c) 2019 amsik
 * Released under the MIT License.
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.EyzyTree = factory());
}(this, (function () { 'use strict';

  var hasChild = function (node) {
      return node.isBatch || node.child.length > 0;
  };

  var EyzyNode = /** @class */ (function () {
      function EyzyNode(nodes, api) {
          this._api = api;
          this._state = api.state;
          this._tree = api.tree;
          this._props = api.tree.props;
          this._nodes = Array.isArray(nodes)
              ? nodes
              : (nodes ? [nodes] : []);
      }
      Object.defineProperty(EyzyNode.prototype, "length", {
          get: function () {
              return this._nodes.length;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(EyzyNode.prototype, "result", {
          get: function () {
              return this._nodes.length ? this._nodes : null;
          },
          enumerable: true,
          configurable: true
      });
      EyzyNode.prototype._operate = function (updateState, operator) {
          var _this = this;
          var result = this._nodes
              .map(function (node) { return operator(node, _this._state); })
              .every(function (res) { return false !== res; });
          if (updateState) {
              this._tree.updateState(this._state);
          }
          return this._nodes.length == 0 ? false : result;
      };
      EyzyNode.prototype.remove = function () {
          var _this = this;
          return this._operate(false, function (node, state) {
              return _this._api.remove(node.id);
          });
      };
      EyzyNode.prototype.empty = function () {
          var _this = this;
          return this._operate(true, function (node, state) {
              if (!hasChild(node)) {
                  return false;
              }
              _this._api.core.clearKeys(node, true);
              state.set(node.id, {
                  child: [],
                  indeterminate: false,
                  isBatch: false
              });
              if (node.parent) {
                  _this._tree.refreshIndeterminateState(node.id, !!node.checked, false);
              }
          });
      };
      EyzyNode.prototype.select = function (extendSelection) {
          var _this = this;
          return this._operate(false, function (node) {
              if (node.selected) {
                  return false;
              }
              if (_this._props.multiple) {
                  _this._tree.select(node, false, extendSelection);
              }
              else {
                  _this._tree.select(node);
              }
          });
      };
      EyzyNode.prototype.unselect = function () {
          var _this = this;
          return this._operate(false, function (node) {
              if (!node.selected) {
                  return false;
              }
              _this._tree.unselect(node);
          });
      };
      EyzyNode.prototype.check = function () {
          var _this = this;
          if (!this._props.checkable) {
              return false;
          }
          return this._operate(false, function (node) {
              if (node.checked) {
                  return false;
              }
              _this._tree.check(node);
          });
      };
      EyzyNode.prototype.uncheck = function () {
          var _this = this;
          if (!this._props.checkable) {
              return false;
          }
          return this._operate(false, function (node, state) {
              if (!node.checked) {
                  return false;
              }
              _this._tree.check(node);
          });
      };
      EyzyNode.prototype.disable = function () {
          return this._operate(true, function (node, state) {
              if (node.disabled) {
                  return false;
              }
              state.set(node.id, 'disabled', true);
          });
      };
      EyzyNode.prototype.enable = function () {
          return this._operate(true, function (node, state) {
              if (!node.disabled) {
                  return false;
              }
              state.set(node.id, 'disabled', false);
          });
      };
      EyzyNode.prototype.disableCheckbox = function () {
          if (!this._props.checkable) {
              return false;
          }
          return this._operate(true, function (node, state) {
              if (node.disabledCheckbox) {
                  return false;
              }
              state.set(node.id, 'disabledCheckbox', true);
          });
      };
      EyzyNode.prototype.enableCheckbox = function () {
          var _this = this;
          if (!this._props.checkable) {
              return false;
          }
          return this._operate(true, function (node, state) {
              if (!node.disabledCheckbox) {
                  return false;
              }
              state.set(node.id, 'disabledCheckbox', false);
              if (hasChild(node) && _this._props.useIndeterminateState) {
                  var iNode = node.checked
                      ? node
                      : node.child[0];
                  _this._tree.refreshIndeterminateState(iNode.id, !!iNode.checked, false);
              }
          });
      };
      EyzyNode.prototype.expand = function () {
          var _this = this;
          return this._operate(false, function (node) {
              if (!hasChild(node) || node.expanded) {
                  return false;
              }
              _this._tree.expand(node);
          });
      };
      EyzyNode.prototype.collapse = function () {
          var _this = this;
          return this._operate(false, function (node) {
              if (!hasChild(node) || !node.expanded) {
                  return false;
              }
              _this._tree.expand(node);
          });
      };
      EyzyNode.prototype.data = function (key, value) {
          var _this = this;
          var nodes = this._nodes;
          if (1 === nodes.length) {
              return this._api.core.data(nodes[0], key, value);
          }
          return nodes.map(function (node) { return _this._api.core.data(node, key, value); });
      };
      EyzyNode.prototype.hasClass = function (className) {
          var _this = this;
          return this._nodes.some(function (node) { return _this._api.core.hasClass(node, className); });
      };
      EyzyNode.prototype.addClass = function () {
          var _this = this;
          var classNames = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              classNames[_i] = arguments[_i];
          }
          this._nodes.forEach(function (node) { return _this._api.core.addClass(node, classNames); });
          return true;
      };
      EyzyNode.prototype.removeClass = function () {
          var _this = this;
          var classNames = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              classNames[_i] = arguments[_i];
          }
          return this._operate(false, function (node) {
              if (!node.className) {
                  return;
              }
              var oldClassNames = node.className;
              var updatedNode = _this._api.core.removeClass(node, classNames);
              return oldClassNames !== updatedNode.className;
          });
      };
      EyzyNode.prototype._find = function (multiple, query) {
          var _this = this;
          var core = this._api.core;
          var nodes = core
              .flatMap(this._nodes)
              .nodes
              .filter(function (node) { return !~_this._nodes.indexOf(node); });
          return core.find(nodes, multiple, query);
      };
      EyzyNode.prototype.find = function () {
          var query = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              query[_i] = arguments[_i];
          }
          var node = this._find(false, query);
          return new EyzyNode(node ? [node] : [], this._api);
      };
      EyzyNode.prototype.findAll = function () {
          var query = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              query[_i] = arguments[_i];
          }
          var nodes = this._find(true, query);
          return new EyzyNode(nodes && nodes.length ? nodes : [], this._api);
      };
      return EyzyNode;
  }());

  var callMethod = function (api, name, criteria, args) {
      return api._operate(criteria, function (node) {
          return args ? node[name].apply(node, args) : node[name]();
      });
  };
  var EyzyTreeAPI = /** @class */ (function () {
      function EyzyTreeAPI(api, isMultiple) {
          if (isMultiple === void 0) { isMultiple = false; }
          this._tree = api.tree;
          this._state = api.state;
          this._api = api;
          this.isMultiple = isMultiple;
      }
      EyzyTreeAPI.prototype.useMultiple = function (isMultiple) {
          this.isMultiple = isMultiple;
          return this;
      };
      EyzyTreeAPI.prototype._operate = function (criteria, operator) {
          var _a, _b;
          if (!Array.isArray(criteria)) {
              criteria = [criteria];
          }
          var nodes = this.isMultiple
              ? (_a = this._api).findAll.apply(_a, criteria) : (_b = this._api).find.apply(_b, criteria);
          if (!nodes || Array.isArray(nodes) && !nodes.length) {
              return false;
          }
          return operator(new EyzyNode(nodes, this._api));
      };
      EyzyTreeAPI.prototype.find = function () {
          var criteria = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              criteria[_i] = arguments[_i];
          }
          var _a;
          var result = (_a = this._api).find.apply(_a, criteria);
          if (!result) {
              return null;
          }
          return new EyzyNode(result, this._api);
      };
      EyzyTreeAPI.prototype.findAll = function () {
          var criteria = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              criteria[_i] = arguments[_i];
          }
          var _a;
          var result = (_a = this._api).findAll.apply(_a, criteria);
          if (!result || !result.length) {
              return null;
          }
          return new EyzyNode(result, this._api);
      };
      EyzyTreeAPI.prototype.remove = function (criteria) {
          return callMethod(this, 'remove', criteria);
      };
      EyzyTreeAPI.prototype.empty = function (criteria) {
          return callMethod(this, 'empty', criteria);
      };
      EyzyTreeAPI.prototype.selected = function () {
          return this._api.selected();
      };
      EyzyTreeAPI.prototype.select = function (criteria, extendSelection) {
          return callMethod(this, 'select', criteria, [extendSelection]);
      };
      EyzyTreeAPI.prototype.unselectAll = function () {
          this._tree.unselectAll();
          return true;
      };
      EyzyTreeAPI.prototype.unselect = function (criteria) {
          return callMethod(this, 'unselect', criteria);
      };
      EyzyTreeAPI.prototype.checked = function (valueConsistsOf, ignoreDisabled) {
          return this._api.checked(valueConsistsOf, ignoreDisabled);
      };
      EyzyTreeAPI.prototype.check = function (criteria) {
          return callMethod(this, 'check', criteria);
      };
      EyzyTreeAPI.prototype.uncheck = function (criteria) {
          return callMethod(this, 'uncheck', criteria);
      };
      // exclusive method that not implemented in component;
      EyzyTreeAPI.prototype.uncheckAll = function () {
          if (!this._tree.props.checkable) {
              return false;
          }
          var tree = this._tree;
          var state = tree.getState();
          tree.checked = tree.checked.filter(function (id) {
              state.set(id, 'checked', false);
              return false;
          });
          tree.indeterminate = tree.indeterminate.filter(function (id) {
              state.set(id, 'indeterminate', false);
              return false;
          });
          tree.updateState(state);
          return true;
      };
      EyzyTreeAPI.prototype.disable = function (criteria) {
          return callMethod(this, 'disable', criteria);
      };
      EyzyTreeAPI.prototype.enable = function (criteria) {
          return callMethod(this, 'enable', criteria);
      };
      EyzyTreeAPI.prototype.disableCheckbox = function (criteria) {
          return callMethod(this, 'disableCheckbox', criteria);
      };
      EyzyTreeAPI.prototype.enableCheckbox = function (criteria) {
          return callMethod(this, 'enableCheckbox', criteria);
      };
      // TODO: expandAll - api.findAll({ expanded: true }).expand()
      EyzyTreeAPI.prototype.expand = function (criteria) {
          return callMethod(this, 'expand', criteria);
      };
      // TODO: collapseAll - api.find(node => new EyzyNodeAPI(node).hasChild())
      EyzyTreeAPI.prototype.collapse = function (criteria) {
          return callMethod(this, 'collapse', criteria);
      };
      EyzyTreeAPI.prototype.data = function (criteria, key, value) {
          return callMethod(this, 'data', criteria, [key, value]);
      };
      EyzyTreeAPI.prototype.hasClass = function (criteria, className) {
          return callMethod(this, 'hasClass', criteria, [className]);
      };
      EyzyTreeAPI.prototype.addClass = function (criteria) {
          var classNames = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              classNames[_i - 1] = arguments[_i];
          }
          return callMethod(this, 'addClass', criteria, classNames);
      };
      EyzyTreeAPI.prototype.removeClass = function (criteria) {
          var classNames = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              classNames[_i - 1] = arguments[_i];
          }
          return callMethod(this, 'removeClass', criteria, classNames);
      };
      return EyzyTreeAPI;
  }());

  return EyzyTreeAPI;

})));
